/* ----------------------------------------------------------------------- */
/*         Fernando Berzal Galiano      ETS Ingeniera Informtica         */
/* ----------------------------------------------------------------------- */
/*                            Bases de datos                               */
/* ----------------------------------------------------------------------- */

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <ctype.h>

#include "FBG_CRT.H"
#include "FBG_KBD.H"
#include "FBG_DATE.H"
#include "FBG_MENU.H"
#include "FBG_UTIL.H"

#include "DB.H"
#include "DB_FRM.H"
#include "DB_DAT.H"
#include "DB_DOS.H"
#include "DB_SRC.H"
#include "DB_SORT.H"

/* ----------------------------------------------------------------------- */
/* ----------------------------------------------------------------------- */

DataBase Catalogo;

/* Estructura de las fichas del catlogo del sistema */

#define CAT_NOMBRE 1
#define CAT_TABLA  2
#define CAT_RF     3
#define CAT_NR     4

/* ----------------------------------------------------------------------- */
/* ----------------------------------------------------------------------- */
/* ----------------------------------------------------------------------- */
/* ----------------------------------------------------------------------- */
/* ----------------------------------------------------------------------- */

void FichaActual (long numero)
{
  Ventana (67, 4, 80, 4);
  TextColor (CYAN);
  PrintF ("Ficha: ");
  TextColor (GRIS);
  PrintF ("%5ld", numero);
  Ventana (1, 6, 80, 23);
}

void BorraFichaActual (void)
{
  Ventana (67, 4, 80, 4);
  cls();
  Ventana (1, 6, 80, 23);
}

/* ----------------------------------------------------------------------- */

void NuevasFichas (DataBase BD)
{
  int         tecla;
  Ficha       f;
  unsigned    cursor = EstadoCursor();
  CommandLine CL     = CreaCL();

  Cursor (OCULTO);
  Presentacion ("Introduccin de fichas");

  NuevoComando (CL, "Guardar");
  NuevoComando (CL, "Borrar");
  NuevoComando (CL, "Otra ms");

  do {

     FichaActual (FichasBD(BD)+1);

     tecla = LeeFicha(BD, &f, CL, "Introduccin de fichas");

     if (TeclaCL(CL,tecla)) {

	switch (tecla) {

	     case 'g':
	     case 'G': GuardaFicha (f);
		       LiberaFicha(&f);
		       tecla = ESCAPE;
		       break;

	     case 'b':
	     case 'B': LiberaFicha(&f);
		       tecla = ESCAPE;
		       break;

	     case 'o':
	     case 'O': GuardaFicha (f);
		       LiberaFicha (&f);

	}

     } else LiberaFicha(&f);

  } while (tecla!=ESCAPE);

  DestruyeCL (&CL);
  BorraFichaActual();
  Cursor (cursor);
}

/* ----------------------------------------------------------------------- */
/* ----------------------------------------------------------------------- */
/* ----------------------------------------------------------------------- */
/* ----------------------------------------------------------------------- */
/* ----------------------------------------------------------------------- */

void EstadoActual (long ficha, long total)
{
  Ventana (67, 2, 80, 4);
  TextColor (CYAN);
  PrintF ("Ficha:  ");
  TextColor (GRIS);
  PrintF ("%5ld", ficha);
  GotoXY (1, 3);
  TextColor (CYAN);
  PrintF ("Fichas: ");
  TextColor (GRIS);
  PrintF ("%5ld", total);
  Ventana (1, 6, 80, 23);
}

void BorraEstadoActual (void)
{
  Ventana (67, 2, 80, 4);
  cls();
  Ventana (1, 6, 80, 23);
}

/* ----------------------------------------------------------------------- */

int MenuCamposReferencia (DataBase BD)
{
  int   i;
  int   campo;
  int   tecla;
  int   referencias = 0;
  Menu  MenuCampos  = CreaMenu();

  for (i=1, tecla='A'; i<=CamposBD(BD); i++,tecla++) {
      if (Tipo_Campo(BD,i)==Referencia) {
	OpcionMenu(MenuCampos,tecla,NombreCampo(BD,i));
	referencias++;
	campo = i;
		}
  }

  if (referencias>1) {
     tecla=EjecutaMenu(MenuCampos);

     if (tecla!=ESCAPE) {
	campo = tecla-'A'+1;
     } else {
	campo = 0;
     }
  }

  DestruyeMenu (&MenuCampos);

  return campo;
}

  /* --- */

Ficha TablaCatalogo (char *ficheroFRM)
{
  int   i;
  long  NumTablas = FichasBD(Catalogo);
  Ficha ficha     = NULL;
  char  cadena[80];

  if (NumTablas>0) {

     i = 1;
     ficha = CargaFicha(Catalogo,i);

     while ( i<=NumTablas ) {

       CadenaCampo(ficha,CAT_TABLA,cadena);
       strupr(cadena);

       if (!strcmp(strupr(ficheroFRM),cadena))
	  return ficha;

       i++;
       LiberaFicha(&ficha);
       if (i<=NumTablas)
	  ficha = CargaFicha(Catalogo,i);
     }
  }

  return ficha;
}

  /* --- */

void Gestor (DataBase BD, Ficha descripcion, long numero);

void Referencias (DataBase BD, Ficha f)
{
  Ficha    DescripcionRef;
  DataBase BDRef;
  int      campo = MenuCamposReferencia(BD);
  char     cadena[80];

  if (campo) {

     DescripcionRef = TablaCatalogo(BDCampoReferencia(BD,campo));

     if (DescripcionRef) {

	CadenaCampo(DescripcionRef,CAT_TABLA,cadena);

	BDRef = CargaBD(cadena);

	if (BDRef) {
	   Gestor (BDRef,DescripcionRef,*(long*)Campo(f,campo));
	   LiberaBD (&BDRef);
	}

	LiberaFicha(&DescripcionRef);
     }
  }
}

/* ----------------------------------------------------------------------- */

void Gestor (DataBase BD, Ficha descripcion, long numero)
{
  int      tecla;
  char     OK[3];
  Ficha    f;
  unsigned cursor = EstadoCursor();

  CommandLine CL         = CreaCL();
  CommandLine CL_Edicion = CreaCL();

  if (FichasBD(BD)>0) {

     Cursor (OCULTO);

	  f = CargaFicha (BD, numero);

     NuevoComando (CL, "Anterior");
     NuevoComando (CL, "Siguiente");
     NuevoComando (CL, "Primera");
     NuevoComando (CL, "Ultima");
     NuevoComando (CL, "Ir a...");
     NuevoComando (CL, "Editar");

     if ((*(long*)Campo(descripcion,CAT_NR))==0)
	NuevoComando (CL, "Borrar");

     if ((*(long*)Campo(descripcion,CAT_RF))!=0)
	NuevoComando (CL, "Referencia");

     NuevoComando (CL_Edicion, "Actualizar");
     NuevoComando (CL_Edicion, "Cancelar");

	  do {

	Presentacion (NombreBD(BD));
	EstadoActual (numero, FichasBD(BD) );
	tecla = MuestraFicha (f, CL);

	if (TeclaCL(CL,tecla)) {

	   switch (tecla) {

	     case 'a':
	     case 'A': if (numero>1) {
			  numero--;
			  LiberaFicha (&f);
			  f = CargaFicha (BD, numero);
		       }
		       break;

	     case 's':
	     case 'S': if (numero<FichasBD(BD)) {
			  numero++;
			  LiberaFicha (&f);
			  f = CargaFicha (BD, numero);
		       }
		       break;

	     case 'p':
	     case 'P': if (numero!=1) {
			  numero = 1;
			  LiberaFicha (&f);
			  f = CargaFicha (BD, numero);
		       }
		       break;

	     case 'u':
	     case 'U': if (numero!=FichasBD(BD)) {
			  numero = FichasBD(BD);
			  LiberaFicha (&f);
			  f = CargaFicha (BD, numero);
		       }
		       break;

	     case 'i':
	     case 'I': GotoXY (1, 18);
		       clrLine();
		       GotoXY (28, 18);
		       TextColor (CYAN);
		       PrintF ("Nmero de ficha: ");
		       TextColor (GRIS);

		       do {
			  GotoXY (45,18);
			  EditaEntero (&numero);
		       } while ((numero<0)||(numero>FichasBD(BD)));

		       LiberaFicha (&f);
		       f = CargaFicha (BD, numero);
		       break;

	     case 'b':
	     case 'B': GotoXY (1, 18);
		       clrLine ();
		       GotoXY (17, 18);
		       TextColor (CYAN);
		       PrintF (" Est seguro de querer borrar esta ficha ? ");
		       TextColor (GRIS);
		       strcpy (OK,"No");

		       if (EditaCadena (OK, 3)!=ESCAPE) {
			  if (!strcmp(strupr(OK),"S")||!strcmp(strupr(OK),"SI")) {
			     BorraFicha (&f);

			     if (FichasBD(BD)>0) {
				if (numero>FichasBD(BD))
				   numero = FichasBD(BD);
				f = CargaFicha (BD, numero);
			     }
			  }
		       }
		       break;

	     case 'e':
	     case 'E': tecla = EditaFicha (f, CL_Edicion, "Edicin de la ficha");

		       if (tecla=='a' || tecla=='A') {
			  GuardaFicha (f);
		       } else {
			  LiberaFicha (&f);
			  f = CargaFicha (BD, numero);
		       }

		       tecla = ' ';
		       break;

	     case 'r':
	     case 'R': Referencias(BD,f);
		       break;
	   }
	}

     } while ( tecla!=ESCAPE && FichasBD(BD)>0 );

     if (FichasBD(BD)>0)
	LiberaFicha (&f);

     BorraEstadoActual();
  }

  DestruyeCL (&CL);
  DestruyeCL (&CL_Edicion);
  Cursor (cursor);
}

/* ----------------------------------------------------------------------- */
/* ----------------------------------------------------------------------- */
/* ----------------------------------------------------------------------- */
/* ----------------------------------------------------------------------- */
/* ----------------------------------------------------------------------- */

void EstadoBusqueda (long ficha, long total)
{
  Ventana (1, 4, 14, 4);
  TextColor (GRIS);
  PrintF ("%5ld", ficha);
  TextColor (CYAN);
  PrintF (" / ");
  TextColor (GRIS);
  PrintF ("%-5ld", total);
  Ventana (1, 6, 80, 23);
}

void BorraEstadoBusqueda (void)
{
  Ventana (1, 4, 14, 4);
  cls();
  Ventana (1, 6, 80, 23);
}

/* ----------------------------------------------------------------------- */

  char     cadena[30];                                 /* Tema de bsqueda */
  long     entero;
  double   real;
  long     fecha;
  short    enumerado;
  long     referencia;


void *TemaBusqueda (DataBase BD, int *posicion)
{
  void       *tema = NULL;
  int         tecla;
  int         campo;
  int         HotKey = 'A';
  Menu        MenuCampos = CreaMenu();

  char *Tipo[] = { "Entero",
		   "Real",
		   "Enumerado",
		   "Fecha",
		   "Cadena",
		   "MEMO",
		   "Referencia",
		   "---"
		 };

  /* Tema de bsqueda: Inicializacin */

  strcpy (cadena,"");
  entero = 0;
  real = 0;
  fecha = Hoy();
  enumerado = 1;


  if (CamposBD(BD)>0) {

     for (campo=1; campo<=CamposBD(BD); campo++, HotKey++)
	 OpcionMenu (MenuCampos, HotKey, NombreCampo(BD,campo) );

     Presentacion ("Campo de bsqueda");
     tecla = EjecutaMenu (MenuCampos);

     if (tecla!=ESCAPE) {

	  cls();
	  campo = tecla-'A'+1;
	  Presentacion ("Tema de bsqueda");
	  GotoXY (5, 5);
	  TextColor (CYAN);
	  PrintF ("Campo: ");
	  TextColor (GRIS);
	  PrintF ("%s", NombreCampo(BD,campo));
	  GotoXY (5, 7);
	  TextColor (CYAN);
	  PrintF ("Tipo: ");
	  TextColor (GRIS);
	  PrintF ("%s", Tipo[Tipo_Campo(BD,campo)]);
	  GotoXY (5, 10);
	  TextColor (CYAN);
	  PrintF ("Buscar... ");
	  TextColor (GRIS);

	  switch (Tipo_Campo(BD,campo)) {

	    case Cadena:
	    case MEMO:      do GotoXY (5, 12);
			    while (LeeCadena(cadena,30)==ESCAPE);
			    tema = &cadena;
			    break;

	    case Entero:    do {
				GotoXY (5, 12);
				EditaEntero (&entero);
			    } while ( !CheckCampoEntero (BD, campo, entero) );

			    tema = &entero;
			    break;

	    case Real:      do {
			       GotoXY (5,12);
			       EditaReal (&real);
			    } while ( !CheckCampoReal (BD, campo, real) );

			    tema = &real;
			    break;

	    case Fecha:     do {
				GotoXY (5,12);
				EditaFecha (&fecha);
			    } while (!CheckCampoFecha (BD, campo, fecha) );

			    tema = &fecha;
			    break;

	    case Enumerado: GotoXY (5,12);
			    EditaEnumerado (BD, campo, &enumerado);
			    tema = &enumerado;
			    break;

	    case Referencia:referencia = 0;
			    if (BuscaReferencia(BD,campo,&referencia,"") != ESCAPE) {
			       tema = &referencia;
			    }
	  }
     }
  }

  DestruyeMenu (&MenuCampos);
  *posicion = campo;
  return tema;
}


/* ----------------------------------------------------------------------- */

void BuscaFichas (DataBase BD)
{
  int       campo;
  void     *tema;
  Ficha     f;

  Busqueda  search;
  Busqueda  aux;
  Nodo      nodo;
  Nodo      tmp;

  long      posicion;
  long      numFicha;
  long      relativa;

  unsigned     cursor = EstadoCursor();
  int          tecla;
  CommandLine  CL;


  tema = TemaBusqueda (BD, &campo);

  if (tema) {

     Presentacion ("Resultado de la bsqueda");
     Cursor (OCULTO);
     GotoXY (28, 10);
     TextColor (CYAN);
     Parpadeo (ON);
     PrintF ("Procesando la bsqueda ...");
     Parpadeo (OFF);
     TextColor (GRIS);
     search = BuscaCampo (BD, campo, tema);
     cls ();

     if (search->elementos > 0) {

	 CL = CreaCL();
	 NuevoComando (CL, "Anterior");
	 NuevoComando (CL, "Siguiente");
	 NuevoComando (CL, "Primera");
	 NuevoComando (CL, "Ultima");
	 NuevoComando (CL, "Buscar");

	 nodo = InicioLista (search->fichas);
	 numFicha = *(long *)LeeNodoLista (search->fichas, nodo);
	 relativa = 1;
	 f = CargaFicha (BD, numFicha);

	 do {

	    EstadoActual   (numFicha, FichasBD(BD));
	    EstadoBusqueda (relativa, search->elementos);
	    tecla = MuestraFicha (f, CL);

	    if (TeclaCL(CL,tecla)) {

	       switch (tecla) {

		 case 'a':
		 case 'A': if (nodo!=InicioLista (search->fichas)) {
			      LiberaFicha (&f);
			      nodo = AnteriorLista (search->fichas, nodo);
			      numFicha = *(long *)LeeNodoLista (search->fichas, nodo);
			      relativa--;
			      f = CargaFicha (BD, numFicha);
			   }
			   break;

		 case 's':
		 case 'S': if (nodo!=UltimoLista(search->fichas)) {
			      LiberaFicha (&f);
			      nodo = SiguienteLista (search->fichas, nodo);
			      numFicha = *(long*)LeeNodoLista (search->fichas, nodo);
			      relativa++;
			      f = CargaFicha (BD, numFicha);
			   }
			   break;

		 case 'p':
		 case 'P': if (nodo!=InicioLista(search->fichas)) {
			      LiberaFicha (&f);
			      nodo = InicioLista (search->fichas);
			      numFicha = *(long*)LeeNodoLista (search->fichas, nodo);
			      relativa = 1;
			      f = CargaFicha (BD, numFicha);
			   }
			   break;

		 case 'u':
		 case 'U': if (nodo!=UltimoLista(search->fichas)) {
			      LiberaFicha (&f);
			      nodo = UltimoLista (search->fichas);
			      numFicha = *(long*)LeeNodoLista (search->fichas, nodo);
			      relativa = search->elementos;
			      f = CargaFicha (BD, numFicha);
			   }
			   break;

		 case 'b':
		 case 'B': BorraEstadoActual();
			   BorraEstadoBusqueda();
			   tema = TemaBusqueda (BD, &campo);

			   if (tema) {
			      Presentacion ("Resultado de la bsqueda");
			      Cursor (OCULTO);
			      GotoXY (28, 10);
			      TextColor (CYAN);
			      Parpadeo (ON);
			      PrintF ("Procesando la bsqueda ...");
			      Parpadeo (OFF);
			      TextColor (GRIS);
			      aux = BuscaCampo (BD, campo, tema);
			      cls ();

			      /* Fusiona 'search' con 'aux' */

			      nodo = InicioLista (search->fichas);
			      tmp  = InicioLista (aux->fichas);

			      while (nodo!=FinLista(search->fichas)) {

				 numFicha = *(long*)LeeNodoLista(search->fichas, nodo);

				 if (tmp!=FinLista(aux->fichas))
				    do {
				       posicion = *(long*)LeeNodoLista(aux->fichas, tmp);
				       tmp = SiguienteLista (aux->fichas, tmp);
				    } while (  posicion<numFicha
					    && tmp!=FinLista(aux->fichas) );

				 if (posicion==numFicha) {
				    nodo = SiguienteLista (search->fichas, nodo);
				 } else {
				    tmp = AnteriorLista (aux->fichas, tmp);
				    BorraNodoLista (search->fichas, nodo);
				    search->elementos--;
				 }
			      }

			      if (search->elementos>0) {
				 LiberaFicha (&f);
				 nodo     = InicioLista (search->fichas);
				 numFicha = *(long*)LeeNodoLista (search->fichas, nodo);
				 relativa = 1;
				 f        = CargaFicha(BD, numFicha);
			      } else {
				 GotoXY (12, 8);
				 PrintF ("No se han encontrado coincidencias en la base de datos");
				 GotoXY (8, 10);
				 PrintF ("Pulse ESCAPE para volver al men principal de la base de datos");

				 do tecla = GetChar();
				 while (tecla != ESCAPE);
			      }

			      DestruyeBusqueda (&aux);
			   }
			   break;
	       }
	    }

	 } while (tecla!=ESCAPE);

	 LiberaFicha (&f);
	 BorraEstadoActual();
	 BorraEstadoBusqueda();
	 DestruyeCL (&CL);
	 Cursor (cursor);

     } else {
	 GotoXY (12, 8);
	 PrintF ("No se han encontrado coincidencias en la base de datos");
	 GotoXY (8, 10);
	 PrintF ("Pulse ESCAPE para volver al men principal de la base de datos");

	 do tecla = GetChar();
	 while (tecla != ESCAPE);
     }

     DestruyeBusqueda (&search);
     Cursor (cursor);
  }
}

/* ----------------------------------------------------------------------- */
/* ----------------------------------------------------------------------- */
/* ----------------------------------------------------------------------- */

int MenuCamposOrdenacion (DataBase BD)
{
  int   i;
  int   campo;
  int   tecla;
  Menu  MenuCampos = CreaMenu();

  Presentacion ("Campo por el que ordenar las fichas");

  for (i=1, tecla='A'; i<=CamposBD(BD); i++,tecla++)
      if (Tipo_Campo(BD,i)!=MEMO)
	 OpcionMenu(MenuCampos,tecla,NombreCampo(BD,i));

  tecla=EjecutaMenu(MenuCampos);

  if (tecla!=ESCAPE) {
     campo = tecla-'A'+1;
  } else {
     campo = 0;
  }

  DestruyeMenu (&MenuCampos);

  return campo;
}

/* --- */

int MenuTipoOrdenacion (void)
{
  int   tecla;
  Menu  MenuTipo = CreaMenu();

  OpcionMenu(MenuTipo,'A',"Ascendente");
  OpcionMenu(MenuTipo,'D',"Descendente");

  Presentacion ("Tipo de ordenacin");

  tecla=EjecutaMenu(MenuTipo);

  DestruyeMenu (&MenuTipo);

  if (tecla=='A')
     return ASCENDENTE;
  else if (tecla=='D')
     return DESCENDENTE;
  else
     return 0;
}

/* --- */

void OrdenaFichas (DataBase BD)
{
  unsigned cursor = EstadoCursor();
  int      campo;
  int      ordenacion;

  Cursor (OCULTO);

  campo = MenuCamposOrdenacion (BD);

  if (campo) {

     ordenacion = MenuTipoOrdenacion();

     if (ordenacion==ASCENDENTE || ordenacion==DESCENDENTE) {
	Presentacion ("Ordenacin de las fichas");
	GotoXY (28, 10);
	TextColor (CYAN);
	Parpadeo (ON);
	PrintF ("Realizando la ordenacin...");
	Parpadeo (OFF);
	TextColor (GRIS);
	OrdenarTabla(BD,campo,ordenacion);
	cls ();
     }
  }

  Cursor (cursor);
}

/* ----------------------------------------------------------------------- */
/* ----------------------------------------------------------------------- */
/* ----------------------------------------------------------------------- */

void GestionaBD (DataBase BD, Ficha descripcion)
{
  int  tecla;
  Menu MenuDBMS = CreaMenu();

  OpcionMenu (MenuDBMS, 'I', "Introduccin de datos");
  OpcionMenu (MenuDBMS, 'A', "Acceso a las fichas");
  OpcionMenu (MenuDBMS, 'B', "Bsqueda de informacin");

  if ((*(long*)Campo(descripcion,CAT_NR))==0)
     OpcionMenu (MenuDBMS, 'O', "Ordenacin de las fichas");

  do {

     Presentacion (NombreBD(BD));
     tecla = EjecutaMenu (MenuDBMS);

     switch (tecla) {

       case 'I': NuevasFichas(BD);
		 break;

       case 'A': if (FichasBD(BD)>0)
		    Gestor (BD,descripcion,1);
		 break;

       case 'B': BuscaFichas (BD);
		 break;

       case 'O': OrdenaFichas (BD);
		 break;
     }

  } while (tecla!=ESCAPE);

  DestruyeMenu(&MenuDBMS);
}

/* ----------------------------------------------------------------------- */
/* ----------------------------------------------------------------------- */
/* ----------------------------------------------------------------------- */
/* ----------------------------------------------------------------------- */
/* ----------------------------------------------------------------------- */

int EligeTabla (DataBase Catalogo)
{
  int   i;
  int   tabla;
  int   tecla;
  Menu  MenuBDs   = CreaMenu();
  long  NumTablas = FichasBD(Catalogo);
  Ficha ficha;
  char  cadena[80];

  if (NumTablas>0) {

     for (i=1,tecla='A'; i<=NumTablas; i++,tecla++) {
	 ficha = CargaFicha(Catalogo,i);
	 CadenaCampo(ficha,CAT_NOMBRE,cadena);
	 OpcionMenu(MenuBDs,tecla,cadena);
	 LiberaFicha(&ficha);
     }

     tecla=EjecutaMenu(MenuBDs);

     if (tecla!=ESCAPE) {
	tabla = tecla-'A'+1;
     } else {
	tabla = 0;
     }

  } else {
     tabla = 0;
  }

  DestruyeMenu (&MenuBDs);

  return tabla;
}

/* ----------------------------------------------------------------------- */

void DBMS (DataBase Catalogo)
{
  int      tecla;
  int      tabla;
  long     tablas = FichasBD(Catalogo);
  char     cadena[80];
  Ficha    descripcion;
  DataBase BD;

  if (!tablas) {

     Presentacion ("Bases de Datos");
     cls();
     GotoXY (21, 10);
     PrintF ("No existe ninguna base de datos creada");
     Cursor (OCULTO);

     do
       tecla = GetChar();
     while ((tecla!=ESCAPE)&&(tecla!=RETURN)&&(tecla!=' '));

     Cursor (VISIBLE);

  } else if (tablas==1) {

     tabla = 1;
     descripcion = CargaFicha (Catalogo,tabla);
     CadenaCampo(descripcion,CAT_TABLA,cadena);
     BD = CargaBD(cadena);

     if (BD) {
	GestionaBD (BD,descripcion);
	LiberaBD  (&BD);
     }

     LiberaFicha(&descripcion);

  } else {

     Presentacion ("Bases de datos");
     tabla = EligeTabla(Catalogo);

     while (tabla) {
	   descripcion = CargaFicha (Catalogo,tabla);
	   CadenaCampo(descripcion,CAT_TABLA,cadena);
	   BD = CargaBD(cadena);

	   if (BD) {
	      GestionaBD (BD,descripcion);
	      LiberaBD  (&BD);
	   }

	   LiberaFicha(&descripcion);

	   Presentacion ("Bases de datos");
	   tabla = EligeTabla(Catalogo);
     }
  }
}

/* ----------------------------------------------------------------------- */
/* ----------------------------------------------------------------------- */
/* ----------------------------------------------------------------------- */
/* ----------------------------------------------------------------------- */
/* ----------------------------------------------------------------------- */

DataBase AbreCatalogo (void)
{
  DataBase Catalogo = CreaBD("Catlogo","DBMS.FRM","DBMS.FBG");

  CampoCadena ( Catalogo, "Nombre", 60  );
  CampoCadena ( Catalogo, "Tabla",  60  );
  CampoEntero ( Catalogo, "RF", 0, 1000 );      /* Referencias de la tabla */
  CampoEntero ( Catalogo, "NR", 0, 1000 );      /* Referencias a la tabla  */

  return Catalogo;
}

/* ----------------------------------------------------------------------- */

void CierraCatalogo (DataBase *Catalogo)
{
  LiberaBD(Catalogo);
}

/* ----------------------------------------------------------------------- */

int main (void)
{
  cls();

  // RGB ("Gestor de bases de datos");
  // AnulaBufferTeclado();

  Catalogo = AbreCatalogo();

  DBMS (Catalogo);

  CierraCatalogo(&Catalogo);

  Fin ();
  return 0;
}

/* ----------------------------------------------------------------------- */
