/* ----------------------------------------------------------------------- */
/*         Fernando Berzal Galiano       ETS Ingeniera Informtica        */
/* ----------------------------------------------------------------------- */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <stdarg.h>

#include "DB.H"
#include "DB_FRM.H"

/* Cabecera de un fichero de base de datos */

#define HEADER "DBMS FBGVG"

/* ----------------------------------------------------------------------- */
/*     Informacin necesaria acerca de un campo de un tipo determinado     */
/* ----------------------------------------------------------------------- */
/* ----------------------------------------------------------------------- */

char *Tipo[] = { "Entero",
		 "Real",
		 "Enumerado",
		 "Fecha",
		 "Cadena",
		 "MEMO",
		 "Referencia",
		 "---"
	       };

/* ----------------------------------------------------------------------- */

typedef struct campo {
		       TipoCampo tipo;
		       char      nombre[CAMPO];
		       int       offset;
		       void     *info;
		     } Campo;

/* ----------------------------------------------------------------------- */

typedef struct CEntero {
			 long min;
			 long max;
		       } InfoCampoEntero;

typedef struct CFecha {
			long min;
			long max;
		      } InfoCampoFecha;

typedef struct CReal {
		       double min;
		       double max;
		     } InfoCampoReal;

typedef struct CEnumerado {
			    int elementos;
			    Lista lista;              /* Lista de cadenas */
			  } InfoCampoEnumerado;

typedef struct CCadena {
			 int longitud;
		       } InfoCampoCadena;

typedef struct CMEMO {
		       char fichMEM[FICHERO];
		       char fichNDX[FICHERO];
		     } InfoCampoMEMO;

typedef struct CRef {
		      char RefBD[FICHERO];
		      int  RefPK;
		    } InfoCampoReferencia;

/* ----------------------------------------------------------------------- */
/*      Funciones para el manejo de la estructura de una base de datos     */
/* ----------------------------------------------------------------------- */

char *NombreBD (DataBase BD)
{
  return BD->nombre;
}

int CamposBD (DataBase BD)
{
  return BD->NumCampos;
}


long FichasBD (DataBase BD)
{
  FILE *f = fopen (BD->fichDatos,"a+b");
  long  longitud;

  fseek (f, 0, SEEK_END);
  longitud = ftell (f);
  fclose (f);

  return longitud/(BD->dim);
}


/* ----------------------------------------------------------------------- */
/* ----------------------------------------------------------------------- */

static Campo *LeeInfoCampo (DataBase BD, int posicion)
{
  int i;
  Nodo nodo= ListaVacia(BD->campos)? NULL: InicioLista(BD->campos);

  if ((posicion<=BD->NumCampos)&&(nodo)) {

     for (i=1; i<posicion; i++)
	 nodo = SiguienteLista (BD->campos, nodo);

     return (LeeNodoLista (BD->campos, nodo));

  } else return NULL;
}

/* ----------------------------------------------------------------------- */

char *NombreCampo (DataBase BD, int posicion)
{
 Campo *c=LeeInfoCampo (BD, posicion);

 if (c)
    return c->nombre;
 else
    return NULL;
}

/* ----------------------------------------------------------------------- */

TipoCampo Tipo_Campo (DataBase BD, int posicion)
{
  Campo *c=LeeInfoCampo (BD, posicion);

  if (c)
     return c->tipo;
  else
     return SinTipo;
}

/* ----------------------------------------------------------------------- */

int OffsetCampo (DataBase BD, int posicion)
{
  Campo *c=LeeInfoCampo (BD, posicion);

  if (c)
     return c->offset;
  else
     return -1;
}

/* ----------------------------------------------------------------------- */
/* ----------------------------------------------------------------------- */

int CheckCampoEntero (DataBase BD, int posicion, long valor)
{
  Campo *c=LeeInfoCampo (BD, posicion);

  if (c) {

     if (c->tipo==Entero)
	return (  ((InfoCampoEntero *)(c->info))->min<=valor
	       && ((InfoCampoEntero *)(c->info))->max>=valor );
     else
	return 0;

  } else
     return 0;
}

/* ----------------------------------------------------------------------- */

int CheckCampoFecha (DataBase BD, int posicion, long valor)
{
  Campo *c=LeeInfoCampo (BD, posicion);

  if (c) {

     if (c->tipo==Fecha)
	return (  ((InfoCampoFecha *)(c->info))->min<=valor
	       && ((InfoCampoFecha *)(c->info))->max>=valor );
     else
	return 0;

  } else
     return 0;
}

/* ----------------------------------------------------------------------- */

int CheckCampoReal (DataBase BD, int posicion, double valor)
{
  Campo *c=LeeInfoCampo (BD, posicion);

  if (c) {

     if (c->tipo==Real)
	return (  ((InfoCampoReal *)(c->info))->min<=valor
	       && ((InfoCampoReal *)(c->info))->max>=valor );
     else
	return 0;

  } else
     return 0;
}

/* ----------------------------------------------------------------------- */

long MinCampoEntero (DataBase BD, int posicion)
{
  Campo *c=LeeInfoCampo (BD, posicion);

  if (c) {

     if (c->tipo==Entero)
	return (((InfoCampoEntero *)(c->info))->min);
     else
	return 0;

  } else
     return 0;
}


long MaxCampoEntero (DataBase BD, int posicion)
{
  Campo *c=LeeInfoCampo (BD, posicion);

  if (c) {

     if (c->tipo==Entero)
	return (((InfoCampoEntero *)(c->info))->max);
     else
	return 0;

  } else
     return 0;
}


long MinCampoFecha (DataBase BD, int posicion)
{
  Campo *c=LeeInfoCampo (BD, posicion);

  if (c) {

     if (c->tipo==Fecha)
	return (((InfoCampoFecha *)(c->info))->min);
     else
	return 0;

  } else
     return 0;
}

long MaxCampoFecha (DataBase BD, int posicion)
{
  Campo *c=LeeInfoCampo (BD, posicion);

  if (c) {

     if (c->tipo==Fecha)
	return (((InfoCampoFecha *)(c->info))->max);
     else
	return 0;

  } else
     return 0;
}


double MinCampoReal (DataBase BD, int posicion)
{
  Campo *c=LeeInfoCampo (BD, posicion);

  if (c) {

     if (c->tipo==Real)
	return (((InfoCampoReal *)(c->info))->min);
     else
	return 0;

  } else
     return 0;
}

double MaxCampoReal (DataBase BD, int posicion)
{
  Campo *c=LeeInfoCampo (BD, posicion);

  if (c) {

     if (c->tipo==Real)
	return (((InfoCampoReal *)(c->info))->max);
     else
	return 0;

  } else
     return 0;
}


int LongitudCampoCadena (DataBase BD, int posicion)
{
  Campo *c=LeeInfoCampo (BD, posicion);

  if (c) {

     if (c->tipo==Cadena)
	return (((InfoCampoCadena *)(c->info))->longitud);
     else
	return 0;

  } else
     return 0;
}


Lista ValoresCampoEnumerado (DataBase BD, int posicion)
{
  Campo *c=LeeInfoCampo (BD, posicion);

  if (c) {

     if (c->tipo==Enumerado)
	return (((InfoCampoEnumerado *)(c->info))->lista);
     else
	return NULL;

  } else
     return NULL;
}

int CardinalCampoEnumerado (DataBase BD, int posicion)
{
  Campo *c=LeeInfoCampo (BD, posicion);

  if (c) {

     if (c->tipo==Enumerado)
	return (((InfoCampoEnumerado *)(c->info))->elementos);
     else
	return 0;

  } else
     return 0;
}


char *FicheroCampoMEMO (DataBase BD, int posicion)
{
  Campo *c=LeeInfoCampo (BD, posicion);

  if (c) {

     if (c->tipo==MEMO)
	return (((InfoCampoMEMO *)(c->info))->fichMEM);
     else
	return NULL;

  } else
     return NULL;
}

char *IndiceCampoMEMO  (DataBase BD, int posicion)
{
  Campo *c=LeeInfoCampo (BD, posicion);

  if (c) {

     if (c->tipo==MEMO)
	return (((InfoCampoMEMO *)(c->info))->fichNDX);
     else
	return NULL;

  } else
     return NULL;
}


char *BDCampoReferencia (DataBase BD, int posicion)
{
  Campo *c=LeeInfoCampo (BD, posicion);

  if (c) {

     if (c->tipo==Referencia)
	return (((InfoCampoReferencia *)(c->info))->RefBD);
     else
	return NULL;

  } else
     return NULL;
}

int PKCampoReferencia (DataBase BD, int posicion)
{
  Campo *c=LeeInfoCampo (BD, posicion);

  if (c) {

     if (c->tipo==Referencia)
	return (((InfoCampoReferencia *)(c->info))->RefPK);
     else
	return 0;

  } else
     return 0;
}


/* ----------------------------------------------------------------------- */
/* ----------------------------------------------------------------------- */

void EliminaCampo (DataBase BD, int posicion)
{
  int i, dim;
  Campo *c;
  Nodo nodo= InicioLista(BD->campos);

  if ((posicion<=BD->NumCampos)&&(posicion>0)) {

     for (i=1; i<posicion; i++)
	 nodo = SiguienteLista (BD->campos, nodo);

     c = LeeNodoLista (BD->campos, nodo);

     switch (c->tipo) {

       case Fecha:      dim = sizeof(long int);
			break;

       case Entero:     dim = sizeof(long int);
			break;

       case Real:       dim = sizeof(double);
			break;

       case Cadena:     dim = sizeof (char) * ((InfoCampoCadena *)c->info)->longitud;
			break;

       case MEMO:       dim = 0;
			break;

       case Enumerado:  dim = sizeof (short int);
			DestruyeLista (&(((InfoCampoEnumerado *)c->info)->lista));
			break;

       case Referencia: dim = sizeof (long int);
			break;
     }

     free (c->info);
     BorraNodoLista (BD->campos, nodo);
     BD->dim -= dim;
     BD->NumCampos--;

     while (nodo!=FinLista(BD->campos)) {
	  c = LeeNodoLista (BD->campos,nodo);
	  c->offset -= dim;
	  nodo = SiguienteLista (BD->campos, nodo);
     }
  }
}

/* ----------------------------------------------------------------------- */
/* ----------------------------------------------------------------------- */

static void AgregaCampo ( DataBase BD, char *nombre, TipoCampo tipo, void *info)
{
  Campo c;

  c.tipo   = tipo;
  c.offset = BD->dim;
  strncpy (c.nombre, nombre, CAMPO);

  switch (c.tipo) {

    case Fecha:      BD->dim += sizeof (long int);
		     c.info = malloc (sizeof(InfoCampoFecha));
		     memcpy ( c.info, info, sizeof(InfoCampoFecha));
		     break;

    case Entero:     BD->dim += sizeof (long int);
		     c.info = malloc (sizeof(InfoCampoEntero));
		     memcpy ( c.info, info, sizeof(InfoCampoEntero));
		     break;

    case Real:       BD->dim += sizeof (double);
		     c.info = malloc (sizeof(InfoCampoReal));
		     memcpy ( c.info, info, sizeof(InfoCampoReal));
		     break;

    case Enumerado:  BD->dim += sizeof (short int);
		     c.info = malloc (sizeof(InfoCampoEnumerado));
		     memcpy ( c.info, info, sizeof(InfoCampoEnumerado));
		     break;

    case Cadena:     BD->dim += sizeof (char) * ((InfoCampoCadena *)info)->longitud;
		     c.info = malloc (sizeof(InfoCampoCadena));
		     memcpy ( c.info, info, sizeof(InfoCampoCadena));
		     break;

    case MEMO:	     c.info = malloc (sizeof(InfoCampoMEMO));
		     memcpy (c.info, info, sizeof(InfoCampoMEMO));
		     break;

    case Referencia: BD->dim += sizeof(long int);
		     c.info = malloc (sizeof(InfoCampoReferencia));
		     memcpy (c.info, info, sizeof(InfoCampoReferencia));
		     break;

  }

  InsertaNodoLista (BD->campos, FinLista(BD->campos), &c);
  BD->NumCampos++;
}

/* --- */

void CampoEntero ( DataBase BD, char *nombre, long min, long max)
{
  InfoCampoEntero info;

  info.min = min;
  info.max = max;

  AgregaCampo (BD, nombre, Entero, &info);
}

/* --- */

void CampoReal ( DataBase BD, char *nombre, double min, double max)
{
  InfoCampoReal info;

  info.min = min;
  info.max = max;

  AgregaCampo (BD, nombre, Real, &info);
}

/* --- */

void CampoEnumerado ( DataBase BD, char *nombre )
{
  InfoCampoEnumerado info;

  info.lista = CreaLista (ENUMERADO);

  if (info.lista) {
     info.elementos = 0;
     AgregaCampo (BD, nombre, Enumerado, &info);
  }
}


void ValorCampoEnumerado (DataBase BD, int posicion, char *valor)
{
  int                 pos;
  Nodo                campo = InicioLista (BD->campos);
  InfoCampoEnumerado *ice;

  for (pos=1; pos<posicion; pos++)
      campo = SiguienteLista (BD->campos, campo);

  ice = ((Campo *)LeeNodoLista(BD->campos,campo))->info;

  InsertaNodoLista( ice->lista, FinLista (ice->lista), valor);
  ice->elementos++;
}

/* --- */

void CampoCadena ( DataBase BD, char *nombre, int longitud )
{
  InfoCampoCadena info;

  info.longitud = longitud;

  AgregaCampo (BD, nombre, Cadena, &info);
}

/* --- */

void CampoFecha ( DataBase BD, char *nombre, long min, long max )
{
  InfoCampoFecha info;

  info.min = min;
  info.max = max;

  AgregaCampo (BD, nombre, Fecha, &info);
}

/* --- */

void CampoMEMO ( DataBase BD, char*nombre, char*fMEM, char*fNDX )
{
  InfoCampoMEMO info;

  strncpy (info.fichMEM, fMEM, FICHERO);
  strncpy (info.fichNDX, fNDX, FICHERO);

  AgregaCampo (BD, nombre, MEMO, &info);
}

/* --- */

void CampoReferencia ( DataBase BD, char *nombre, char *RefBD, int RefPK )
{
  InfoCampoReferencia info;

  strncpy (info.RefBD, RefBD, FICHERO);

  info.RefPK = RefPK;

  AgregaCampo (BD, nombre, Referencia, &info);
}

/* ----------------------------------------------------------------------- */
/* ----------------------------------------------------------------------- */

DataBase CreaBD ( char *nombre, char *fichFormato, char *fichDatos )
{
  DataBase BD = malloc (sizeof(InfoDB));

  if (BD) {
     strncpy ( BD->nombre,      nombre,      DATABASE );
     strncpy ( BD->fichFormato, fichFormato, FICHERO  );
     strncpy ( BD->fichDatos,   fichDatos,   FICHERO  );
     BD->campos    = CreaLista (sizeof(Campo));
     BD->NumCampos = 0;
     BD->dim       = 0;
     return BD;
  } else
     return NULL;
}

/* ----------------------------------------------------------------------- */

void LiberaBD (DataBase *BD)
{
  if (*BD) {

     while ((*BD)->NumCampos)
	   EliminaCampo (*BD, (*BD)->NumCampos);

     DestruyeLista (&((*BD)->campos));

     free (*BD);
     *BD = NULL;
  }
}

/* ----------------------------------------------------------------------- */
/*        Volcado del formato de la base de datos a/desde un fichero       */
/* ----------------------------------------------------------------------- */

static void Lee (FILE *fichero, char *cadena, int longitud)
{
  fgets (cadena, longitud, fichero);

  if (cadena[strlen(cadena)-1]=='\n')
     cadena[strlen(cadena)-1] = '\0';
  else while (fgetc(fichero)!='\n')
     ;
}

/* ----------------------------------------------------------------------- */

static void ListaCampoEnumerado (FILE *fichero, InfoCampoEnumerado *info)
{
  Nodo nodo = InicioLista(info->lista);

  fprintf ( fichero, "%d\n", info->elementos);

  while (nodo!=FinLista(info->lista)) {
    fprintf ( fichero, "%s\n", LeeNodoLista(info->lista, nodo) );
    nodo = SiguienteLista(info->lista, nodo);
  }
}

/* ----------------------------------------------------------------------- */

static void CargaCampoEnumerado (FILE *fichero, InfoCampoEnumerado *info)
{
  int  i;
  char cadena[ENUMERADO];

  info->lista = CreaLista (ENUMERADO);

  fscanf ( fichero, "%d\n", &(info->elementos));

  for (i=1; i<=info->elementos; i++) {
      Lee (fichero, cadena, ENUMERADO);
      InsertaNodoLista (info->lista, FinLista(info->lista), cadena);
  }

}

/* ----------------------------------------------------------------------- */

static void GuardaEstructura (DataBase DB, FILE *fichero)
{
  Campo *elemento;
  Nodo   item = InicioLista (DB->campos);

  while (item!=FinLista(DB->campos)) {

    elemento = LeeNodoLista (DB->campos, item);

    fprintf (fichero, "%s\n", elemento->nombre);
    fprintf (fichero, "%s\n", Tipo[elemento->tipo]);

    switch (elemento->tipo) {                         /* Info. adicional */

      case Entero:     fprintf ( fichero, "%ld\n%ld\n",
			  ((InfoCampoEntero *)(elemento->info))->min,
			  ((InfoCampoEntero *)(elemento->info))->max);
		       break;

      case Fecha:      fprintf ( fichero, "%ld\n%ld\n",
			  ((InfoCampoFecha *)(elemento->info))->min,
			  ((InfoCampoFecha *)(elemento->info))->max);
		       break;

      case Real:       fprintf ( fichero, "%lf\n%lf\n",
			  ((InfoCampoReal *)(elemento->info))->min,
			  ((InfoCampoReal *)(elemento->info))->max);
		       break;

      case Cadena:     fprintf ( fichero, "%d\n",
			  ((InfoCampoCadena *)(elemento->info))->longitud);
		       break;

      case MEMO:       fprintf ( fichero, "%s\n%s\n",
			  ((InfoCampoMEMO *)(elemento->info))->fichMEM,
			  ((InfoCampoMEMO *)(elemento->info))->fichNDX);
		       break;

      case Enumerado:  ListaCampoEnumerado (fichero, ((InfoCampoEnumerado *)(elemento->info)));
		       break;

      case Referencia: fprintf ( fichero, "%s\n%d\n",
			  ((InfoCampoReferencia *)(elemento->info))->RefBD,
			  ((InfoCampoReferencia *)(elemento->info))->RefPK);
		       break;
    }

    item = SiguienteLista (DB->campos, item);
  }
}

/* ----------------------------------------------------------------------- */

static void LeeEstructura ( DataBase DB, FILE *fichero)
{
  int   i;
  int   offset = 0;
  Campo NuevoCampo;
  char  tipo[TIPO];

  DB->campos = CreaLista (sizeof(Campo));

  for (i=1; i<=DB->NumCampos; i++) {

      Lee ( fichero, NuevoCampo.nombre, CAMPO);
      Lee ( fichero, tipo, TIPO);

      NuevoCampo.offset = offset;

      if (!strcmp( tipo, Tipo[Entero]))
	 NuevoCampo.tipo = Entero;
      else if (!strcmp( tipo, Tipo[Real]))
	 NuevoCampo.tipo = Real;
      else if (!strcmp( tipo, Tipo[Enumerado]))
	 NuevoCampo.tipo = Enumerado;
      else if (!strcmp( tipo, Tipo[Fecha]))
	 NuevoCampo.tipo = Fecha;
      else if (!strcmp( tipo, Tipo[Cadena]))
	 NuevoCampo.tipo = Cadena;
      else if (!strcmp( tipo, Tipo[MEMO]))
	 NuevoCampo.tipo = MEMO;
      else if (!strcmp( tipo, Tipo[Referencia]))
	 NuevoCampo.tipo = Referencia;

      switch (NuevoCampo.tipo) {

	case Entero:     NuevoCampo.info = malloc(sizeof(InfoCampoEntero));
			 fscanf ( fichero, "%ld\n%ld\n",
			   &(((InfoCampoEntero *)NuevoCampo.info)->min),
			   &(((InfoCampoEntero *)NuevoCampo.info)->max) );
			 offset  += sizeof (long int);
			 break;

	case Real:       NuevoCampo.info = malloc(sizeof(InfoCampoReal));
			 fscanf ( fichero, "%lf\n%lf\n",
			   &(((InfoCampoReal *)NuevoCampo.info)->min),
			   &(((InfoCampoReal *)NuevoCampo.info)->max) );
			 offset  += sizeof (double);
			 break;

	case Fecha:      NuevoCampo.info = malloc(sizeof(InfoCampoFecha));
			 fscanf ( fichero, "%ld\n%ld\n",
			   &(((InfoCampoFecha *)NuevoCampo.info)->min),
			   &(((InfoCampoFecha *)NuevoCampo.info)->max) );
			 offset  += sizeof (long int);
			 break;

	case Cadena:     NuevoCampo.info = malloc(sizeof(InfoCampoCadena));
			 fscanf ( fichero, "%d\n",
			   &(((InfoCampoCadena *)NuevoCampo.info)->longitud) );
			 offset += ((InfoCampoCadena*)NuevoCampo.info)->longitud;
			 break;

	case MEMO:       NuevoCampo.info = malloc (sizeof(InfoCampoMEMO));
			 Lee ( fichero, ((InfoCampoMEMO *)NuevoCampo.info)->fichMEM, FICHERO);
			 Lee ( fichero, ((InfoCampoMEMO *)NuevoCampo.info)->fichNDX, FICHERO);
			 break;

	case Enumerado:  NuevoCampo.info = malloc (sizeof(InfoCampoEnumerado));
			 CargaCampoEnumerado (fichero, ((InfoCampoEnumerado *)(NuevoCampo.info)));
			 offset += sizeof (short int);
			 break;

	case Referencia: NuevoCampo.info = malloc (sizeof(InfoCampoReferencia));
			 Lee ( fichero, ((InfoCampoReferencia *)NuevoCampo.info)->RefBD, FICHERO);
			 fscanf ( fichero, "%d\n", &(((InfoCampoReferencia *)NuevoCampo.info)->RefPK) );
			 offset += sizeof (long int);
			 break;
      }

      InsertaNodoLista (DB->campos, FinLista(DB->campos), &NuevoCampo);
  }

  DB->dim = offset;
}

/* ----------------------------------------------------------------------- */

void AlmacenaBD (DataBase DB)
{
  FILE *fich = fopen(DB->fichFormato, "w");

  if (fich) {

     fprintf (fich,"%s\n", HEADER);
     fprintf (fich,"%s\n", DB->fichDatos);
     fprintf (fich,"%s\n", DB->nombre);
     fprintf (fich,"%d\n", DB->NumCampos);

     GuardaEstructura (DB, fich);

     fclose (fich);
  }

}

/* ----------------------------------------------------------------------- */

DataBase CargaBD ( char *fichero )
{
  char     cabecera[FICHERO];
  char     aux[DATABASE];
  char    *buffer;
  FILE    *fich = fopen(fichero, "r");
  DataBase BD = NULL;

  if (fich) {

     buffer = malloc(BUFFER);

     if (buffer)
	setvbuf (fich, buffer, _IOFBF, BUFFER);

     Lee ( fich, cabecera, FICHERO);

     if (strcmp(cabecera,HEADER)==0) {

	BD = malloc (sizeof(InfoDB));

	if (BD) {

	   strncpy ( BD->fichFormato, fichero, FICHERO);

	   Lee ( fich, BD->fichDatos, FICHERO);

	   Lee ( fich, BD->nombre, DATABASE);

	   Lee ( fich, aux, DATABASE);
	   BD->NumCampos = atoi(aux);

	   LeeEstructura (BD, fich);
	}

     }

     if (buffer)
	free (buffer);

     fclose (fich);
  }

  return BD;
}

/* ----------------------------------------------------------------------- */
