/* ----------------------------------------------------------------------- */
/*         Fernando Berzal Galiano       ETS Ingeniera Informtica        */
/* ----------------------------------------------------------------------- */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "DB.H"
#include "DB_FRM.H"
#include "DB_SEC.H"

/* ----------------------------------------------------------------------- */
/*            Gestin del fichero principal de la base de datos            */
/* ----------------------------------------------------------------------- */

static void *CargaDatoSecuencial (FILE *fich, DataBase BD)
{
  void *dato = malloc(BD->dim);

  if (dato)
     fread (dato, BD->dim, 1, fich);

  return dato;
}

/* ----------------------------------------------------------------------- */
/*                         Gestin de campos MEMO                          */
/* ----------------------------------------------------------------------- */

#ifndef MARCA_MEMO
#define MARCA_MEMO ""
#endif

static void LeeCadSecuencial (FILE *fichero, unsigned char *cadena, int longitud)
{
  fgets (cadena, longitud, fichero);

  if (cadena[strlen(cadena)-1]=='\n')
     cadena[strlen(cadena)-1] = '\0';
  else
     while (!feof(fichero) && fgetc(fichero)!='\n')
	   ;
}

/* ----------------------------------------------------------------------- */
/*  Carga el contenido del campo MEMO de la ficha 'numero'                 */

static Lista CargaMEMOSecuencial (FILE *f)
{
  char  cadena[CAMPOMEMO];
  char *s;
  Lista l = CreaLista(0);

  if (l) {

     do {
	 LeeCadSecuencial (f, cadena, CAMPOMEMO);

	 if (strcmp(cadena,MARCA_MEMO)) {
	    s = (char *) malloc (strlen(cadena)+1);
	    strcpy (s, cadena);
	    InsertaNodoLista (l, FinLista(l), s);
	 }

     } while (strcmp(cadena,MARCA_MEMO));

  }

  return l;
}

/* ----------------------------------------------------------------------- */
/*                     Operaciones bsicas con fichas                      */
/* ----------------------------------------------------------------------- */

Ficha ObtieneSiguienteFicha (Acceso a)
{
  Ficha f        =  NULL;
  int   campo    ;
  int   posicion = 0;
  int   memo     = 1;
  void *dato     ;

  dato = CargaDatoSecuencial (a->f[0],a->bd);

  if (dato) {

     f = malloc (sizeof(struct Ficha));

     if (f) {

	a->numero++;

	f->BD     = a->bd;
	f->numero = a->numero;
	f->dato   = dato;
	f->info   = calloc (a->bd->NumCampos,sizeof(void*));

	for (campo=1; campo<=a->bd->NumCampos; campo++) {

	    switch (Tipo_Campo(a->bd,campo)) {

	      case Fecha:
	      case Referencia:
	      case Entero:     f->info[campo-1] = (char *)dato + posicion;
			       posicion += sizeof (long int);
			       break;

	      case Real:       f->info[campo-1] = (char *)dato + posicion;
			       posicion += sizeof (double);
			       break;

	      case Cadena:     f->info[campo-1] = (char *)dato + posicion;
			       posicion += LongitudCampoCadena(a->bd,campo);
			       break;

	      case Enumerado:  f->info[campo-1] = (char *)dato + posicion;
			       posicion += sizeof (short int);
			       break;

	      case MEMO:       f->info[campo-1] = CargaMEMOSecuencial(a->f[memo]);
			       memo++;
			       break;
	    }
	}

     } else free(dato);
  }

  return f;
}

/* ----------------------------------------------------------------------- */
/* ----------------------------------------------------------------------- */

Acceso InicializaAccesoSecuencial (DataBase BD)
{
  int    campo;
  int    num;
  Acceso a = malloc(sizeof(struct Acceso));

  if (a) {

     a->NumFicheros = 1;

     for (campo=1; campo<=BD->NumCampos; campo++)
	 if (Tipo_Campo(BD,campo)==MEMO)
	    a->NumFicheros++;

     a->bd     = BD;
     a->numero = 0;
     a->f      = calloc (a->NumFicheros,sizeof(FILE *));

     if (a->f) {

	a->f[0]   = fopen(BD->fichDatos,"rb");

	num = 1;

	for (campo=1; campo<=BD->NumCampos; campo++)
	    if (Tipo_Campo(BD,campo)==MEMO) {
	       a->f[num] = fopen(FicheroCampoMEMO(BD,campo),"rb");
	       num++;
	    }

     } else {
	free (a);
	a = NULL;
     }
  }

  return a;
}

/* ----------------------------------------------------------------------- */

void FinalizaAccesoSecuencial (Acceso *a)
{
  int num;

  for (num=0; num<((*a)->NumFicheros); num++)
      fclose ((*a)->f[num]);

  free ((*a)->f);
  free (*a);
  *a = NULL;
}

/* ----------------------------------------------------------------------- */
/* ----------------------------------------------------------------------- */
/* ----------------------------------------------------------------------- */
