/* ----------------------------------------------------------------------- */
/*       Fernando Berzal Galiano          ETS Ingeniera Informtica       */
/* ----------------------------------------------------------------------- */
/*                  Procesamiento de cadenas de caracteres                 */
/* ----------------------------------------------------------------------- */

#include <string.h>

#include "fb_str.h"

/* Tabla de cdigos para la comparacin de caracteres */

#ifndef __TABLA_DE_CARACTERES__
#define __TABLA_DE_CARACTERES__

/* ----------------------------------------------------------------------- */
/* Tabla de caracteres                                                     */
/* CODEPAGE = 437 en MS-DOS                                                */

char Tabla[] = {   0,  0,  0,  0,  0,  0,  0,  0,              /*          */
		   0,  0,  0,  0,  0,  0,  0,  0,              /*          */
		   0,  0,  0,  0,  0,  0,  0,  0,              /*          */
		   0,  0,  0,  0,  0,  0,  0,  0,              /*          */
		   0, 42, 42, 45, 46, 45, 43, 42,              /*  !"#$%&' */
		  42, 42, 45, 45, 41, 42, 41, 42,              /* ()*+,-./ */
		  30, 31, 32, 33, 34, 35, 36, 37,              /* 01234567 */
		  38, 39, 41, 41, 45, 45, 45, 42,              /* 89:;<=>? */
		  48,  1,  2,  3,  4,  5,  6,  7,              /* @ABCDEFG */
		   8,  9, 10, 11, 12, 13, 14, 16,              /* HIJKLMNO */
		  17, 18, 19, 20, 21, 22, 23, 24,              /* PQRSTUVW */
		  25, 26, 27, 42, 42, 42, 45,  0,              /* XYZ[\]^_ */
		  42,  1,  2,  3,  4,  5,  6,  7,              /* `abcdefg */
		   8,  9, 10, 11, 12, 13, 14, 16,              /* hijklmno */
		  17, 18, 19, 20, 21, 22, 23, 24,              /* pqrstuvw */
		  25, 26, 27, 42, 43, 42, 45, 49,              /* xyz{|}~ */
		   3, 22,  5,  1,  1,  1,  1,  3,              /*  */
		   5,  5,  5,  9,  9,  9,  1,  1,              /*  */
		   5, 40, 40, 16, 16, 16, 22, 22,              /*  */
		  26, 16, 22, 46, 46, 46, 46, 46,              /*  */
		   1,  9, 16, 22, 15, 15, 42, 42,              /*  */
		  42, 49, 49, 48, 48, 42, 42, 42,              /*  */
		  49, 49, 49, 49, 49, 49, 49, 49,              /*  */
		  49, 49, 49, 49, 49, 49, 49, 49,              /*  */
		  49, 49, 49, 49, 49, 49, 49, 49,              /*  */
		  49, 49, 49, 49, 49, 49, 49, 49,              /*  */
		  49, 49, 49, 49, 49, 49, 49, 49,              /*  */
		  49, 49, 49, 49, 49, 49, 49, 49,              /*  */
		  47, 47, 47, 47, 47, 47, 47, 47,              /*  */
		  47, 47, 47, 48, 48, 48, 48, 48,              /*  */
		  45, 45, 45, 45, 48, 48, 45, 45,              /*  */
		  48, 48, 48, 48, 48, 48, 49,  0  };           /*  */

/* ----------------------------------------------------------------------- */

#endif

/* ----------------------------------------------------------------------- */
/*  Comparacin de cadenas                                                 */
/* ----------------------------------------------------------------------- */

int CadCmp (unsigned char *c1, unsigned char *c2)
{
  int i  = 0,
      L1 = strlen(c1),
      L2 = strlen(c2),
      L  = (L1>L2)? L2: L1;

  if (L1==0 || L2==0) {
     if (L1==0 && L2!=0)
	return -1;
     else if (L1!=0 && L2==0)
	return 1;
     else if (L1==0 && L2==0)
	return 0;
  }

  while ( Tabla[c1[i]] == Tabla[c2[i]] && i<L )
	i++;

  if (i==L) {

     if (L1>L2)
	return 1;
     else if (L1<L2)
	return -1;
     else
	return 0;

  } else if ( Tabla[c1[i]] < Tabla[c2[i]] ) {

     return -1;

  } else {

     return 1;

  }

}

/* ----------------------------------------------------------------------- */
/*  Bsqueda de subcadenas                                                 */
/* ----------------------------------------------------------------------- */

/* ----------------------------------------------------------------------- */
/*  Fuerza bruta: O(M*N)                                                   */
/* ----------------------------------------------------------------------- */

int FB (unsigned char *subcadena, unsigned char *cadena)
{
  int i,
      j,
      M = strlen(subcadena),
      N = strlen(cadena);

  for (i=0, j=0; j<M && i<N; i++, j++)
      while ( Tabla[cadena[i]] != Tabla[subcadena[j]] ) {
	    i -= j-1;
	    j  = 0;
      }

  if ((j==M)&&(i-M<N))
     return i-M;
  else
     return -1;
}

/* ----------------------------------------------------------------------- */
/*  Algoritmo de Knuth, Morris y Pratt: O(M+N)                             */
/* ----------------------------------------------------------------------- */

int next[256];                    /* Matriz para la construccin de la FSM */

 /* ----- */

int KMP (unsigned char *subcadena, unsigned char *cadena)
{
  int i,
      j,
      M = strlen (subcadena),
      N = strlen (cadena);

  /* Inicializacin de la matriz NEXT */

  next[0] = -1;

  for ( i=0, j=-1; i<M; i++, j++, next[i]=j )
      while ( (j>=0) && ( Tabla[subcadena[i]] != Tabla[subcadena[j]] ) )
	    j = next[j];

  /* Aplicacin del algoritmo */

  for (i=0, j=0; j<M && i<N; i++, j++)
      while ( (j>=0) && ( Tabla[cadena[i]] != Tabla[subcadena[j]] ) )
	    j = next[j];

  if (j==M)
     return i-M;
  else
     return -1;
}

/* ----------------------------------------------------------------------- */

void AutomataKMP (unsigned char *subcadena)
{
  int i,
      j,
      M = strlen (subcadena);

  /* Inicializacin de la matriz NEXT */

  next[0] = -1;

  for ( i=0, j=-1; i<M; i++, j++, next[i]=j )
      while ( (j>=0) && ( Tabla[subcadena[i]] != Tabla[subcadena[j]] ) )
	    j = next[j];
}

/* ----------------------------------------------------------------------- */

int BusquedaKMP (unsigned char *subcadena, unsigned char *cadena)
{
  int i,
      j,
      M = strlen (subcadena),
      N = strlen (cadena);

  for (i=0, j=0; j<M && i<N; i++, j++)
      while ( (j>=0) && ( Tabla[cadena[i]] != Tabla[subcadena[j]] ) )
	    j = next[j];

  if (j==M)
     return i-M;
  else
     return -1;
}

/* ----------------------------------------------------------------------- */
/*  Algoritmo de Boyer-Moore: O(M+N)                                       */
/* ----------------------------------------------------------------------- */

char skip[50];

 /* ----- */

int BM (unsigned char *subcadena, unsigned char *cadena)
{
  int i,
      j,
      t,
      M = strlen (subcadena),
      N = strlen (cadena);

  /* Inicializacin de la matriz SKIP */

  for (i=0; i<50; i++)
      skip[i] = M;

  for (j=0; j<M; j++)
      skip[Tabla[subcadena[j]]] = M-j-1;

  /* Aplicacin del algoritmo */

  for (i=M-1, j=M-1; j>=0 && i<N; i--, j--)
      while ( Tabla[cadena[i]] != Tabla[subcadena[j]] ) {
	    t = skip[Tabla[cadena[i]]];
	    i+= ((M-j>t)? M-j: t);
	    j = M-1;
      }

  if (i<N)
     return i+1;
  else
     return -1;
}

/* ----------------------------------------------------------------------- */
/*  Algoritmo de Rabin-Karp: O(N*M) en casos increbles                    */
/* ----------------------------------------------------------------------- */

#define q 33554393

int RK (unsigned char *subcadena, unsigned char *cadena)
{
  int i;

  int M = strlen(subcadena),
      N = strlen(cadena);

  long int dM = 1,
	   h1 = 0,
	   h2 = 0;

  for (i=1; i<M; i++)
      dM = (dM*50) % q;

  for (i=0; i<M; i++) {
      h1 = (h1*50+Tabla[subcadena[i]]) % q;
      h2 = (h2*50+Tabla[cadena[i]]) % q;
  }

  for (i=0; h1!=h2 && i<=N-M; i++) {
      h2 = (h2+q*50-Tabla[cadena[i]]*dM) % q;
      h2 = (h2*50+Tabla[cadena[i+M]]) % q;
  }

  if (i>N-M)
     return -1;
  else
     return i;
}

/* ----------------------------------------------------------------------- */
/* ----------------------------------------------------------------------- */
/*  Bsqueda con expresiones regulares:  O ( M * N^2 )                     */
/* ----------------------------------------------------------------------- */
/*  <expresion> ::= <termino> | <termino>+<expresion>                      */
/*  <termino>   ::= <factor> | <factor><termino>                           */
/*  <factor>    ::= (<expresion>) | letra | (<expresion>)* | letra*        */


/*  Autmata                                                               */
/*  - Cada estado tiene una etiqueta y dos transiciones                    */

#define MAX_ESTADOS 255

int ch[MAX_ESTADOS+1];
int t1[MAX_ESTADOS+1];
int t2[MAX_ESTADOS+1];

void Estado (int estado, unsigned char etiqueta, int estado1, int estado2)
{
  ch[estado] = etiqueta;
  t1[estado] = estado1;
  t2[estado] = estado2;
}

/* ----------------------------------------------------------------------- */
/*  Top-down parser:                                                       */
/*  Se encarga de construir el autmata a partir de una expresin regular. */
/* ----------------------------------------------------------------------- */

int expresion (void);
int termino (void);
int factor (void);


int j;
int error;
int estado_actual;
unsigned char *p;


int letra (unsigned char chr)
{
  return ( chr!='\0' && chr!='+' && chr!='*' && chr!='(' && chr!=')' );
}


int factor (void)
{
  int tmp1, tmp2, r;

  if (!error) {

     tmp1 = estado_actual;

     if (p[j]=='(') {

	j++;
	tmp2 = expresion ();

	if (p[j]==')')
	   j++;
	else
	   error = 1;

     } else if (letra(p[j])) {
	Estado (estado_actual, Tabla[p[j]], estado_actual+1, estado_actual+1 );
	tmp2 = estado_actual;
	j++;
	estado_actual++;
     } else
	error = 2;

     if (p[j]=='*') {
	Estado (estado_actual, 'X', estado_actual+1, tmp2);
	r = estado_actual;
	t1[tmp1-1] = estado_actual;
	j++;
	estado_actual++;
     } else
	r = tmp2;
  }

  return r;
}


int termino (void)
{
  int r;

  if (!error) {

     r = factor ();

     if ( (p[j]=='(') || letra(p[j]) )
	(void) termino ();
  }

  return r;
}


int expresion (void)
{
  int tmp1, tmp2, r;

  if (!error) {

     tmp1 = termino ();
     r    = tmp1;

     if (p[j]=='+') {
	j++;
	estado_actual++;
	r = tmp2 = estado_actual;
	estado_actual++;
	Estado (tmp2, 'X', expresion(), tmp1 );
	Estado (tmp2-1, 'X', estado_actual, estado_actual );
     }
  }

  return r;
}


int Parser_Constructor (unsigned char *cadena)
{
  j = 0;
  error = 0;
  estado_actual = 1;
  p = cadena;

  t1[0] = expresion();

  Estado ( 0, 'X', t1[0], t1[0]);
  Estado (estado_actual, 'X', 0, 0);

  return (!error);
}

/* ----------------------------------------------------------------------- */
/*  Deque                                                                  */
/* ----------------------------------------------------------------------- */

#include "fb_list.h"

Lista deque;

void DQ_Inicializa (void)
{
  deque = CreaLista (sizeof(int));
}

void DQ_Destruye (void)
{
  DestruyeLista (&deque);
}

int DQ_Vacia (void)
{
  return (ListaVacia(deque));
}

void DQ_push (int estado)
{
  InsertaNodoLista (deque, InicioLista(deque), &estado);
}

int DQ_pop (void)
{
  int aux;

  aux = *(int*)LeeNodoLista (deque, InicioLista(deque));
  BorraNodoLista (deque, InicioLista(deque));

  return aux;
}

void DQ_put (int estado)
{
  InsertaNodoLista (deque, FinLista(deque), &estado);
}

/* ----------------------------------------------------------------------- */
/*  Bsqueda de patrones con expresiones regulares                         */
/* ----------------------------------------------------------------------- */

#define SCAN  -1

int match (unsigned char *cadena)
{
  int fallo  = 0,
      j      = 0,
      N      = strlen(cadena),
      estado = t1[0];

  DQ_Inicializa();
  DQ_put (SCAN);

  while (estado && !fallo ) {

    if ( j<N && estado==SCAN) {

       j++;
       DQ_put ( SCAN );

    } else if ( ch[estado]=='X' ) {                         /* Estado nulo */

       DQ_push (t1[estado]);

       if (t1[estado]!=t2[estado])
	  DQ_push (t2[estado]);

    } else if ( j<N && ch[estado]==Tabla[cadena[j]] ) {

	  DQ_put (t1[estado]);
    }

    if (DQ_Vacia())
       fallo = 1;
    else
       estado = DQ_pop ();
  }

  DQ_Destruye();

  if (!estado)
     return j;
  else
     return -1;
}

/* ----------------------------------------------------------------------- */
/*  Bsqueda en una cadena de caracteres utilizando expresiones regulares  */
/* ----------------------------------------------------------------------- */

int ER (unsigned char *expresion_regular, unsigned char *cadena)
{
  int pos,
      OK;

  if (Parser_Constructor (expresion_regular)) {

     for ( pos=0, OK=-1; *cadena!='\0' && OK<0; cadena++, pos++)
	 OK = match (cadena);

     if (OK>=0)
	return pos-1;
     else
	return -1;

  } else
     return -2;
}

/* ----------------------------------------------------------------------- */