/* ----------------------------------------------------------------------- */
/*       Fernando Berzal Galiano          ETS Ingeniera Informtica       */
/* ----------------------------------------------------------------------- */
/*                  Procesamiento de cadenas de caracteres                 */
/* ----------------------------------------------------------------------- */

#ifndef __FB_CADENAS__

#define __FB_CADENAS__

/* ----------------------------------------------------------------------- */
/*  Comparacin de cadenas                                                 */
/* ----------------------------------------------------------------------- */

int CadCmp (unsigned char *c1, unsigned char *c2);


/* ----------------------------------------------------------------------- */
/*  Bsqueda de subcadenas                                                 */
/* ----------------------------------------------------------------------- */

/* ----------------------------------------------------------------------- */
/*  Fuerza bruta: O(M*N)                                                   */
/* ----------------------------------------------------------------------- */

int FB (unsigned char *subcadena, unsigned char *cadena);

/* ----------------------------------------------------------------------- */
/*  Algoritmo de Knuth, Morris y Pratt: O(M+N)                             */
/* ----------------------------------------------------------------------- */

int KMP (unsigned char *subcadena, unsigned char *cadena);

void AutomataKMP (unsigned char *subcadena);
int  BusquedaKMP (unsigned char *subcadena, unsigned char *cadena);

/* ----------------------------------------------------------------------- */
/*  Algoritmo de Boyer-Moore: O(M+N)                                       */
/* ----------------------------------------------------------------------- */

int BM (unsigned char *subcadena, unsigned char *cadena);

/* ----------------------------------------------------------------------- */
/*  Algoritmo de Rabin-Karp: O(N*M) en casos increbles                    */
/* ----------------------------------------------------------------------- */

int RK (unsigned char *subcadena, unsigned char *cadena);


/* ----------------------------------------------------------------------- */

/* ----------------------------------------------------------------------- */
/*  Bsqueda con expresiones regulares:  O ( M * N^2 )                     */
/* ----------------------------------------------------------------------- */
/*  <expresion> ::= <termino> | <termino>+<expresion>                      */
/*  <termino>   ::= <factor> | <factor><termino>                           */
/*  <factor>    ::= (<expresion>) | letra | (<expresion>)* | letra*        */

/* ----------------------------------------------------------------------- */
/*  Bsqueda de patrones con expresiones regulares                         */
/* ----------------------------------------------------------------------- */

int ER (unsigned char *expresion_regular, unsigned char *cadena);

/* ----------------------------------------------------------------------- */

#endif